/** \file tickTime.h
 *
 *  Definitions of some timing and tick interrupt functions
 *
 *  Copyright (c) 2014, 2015  Albrecht Weinert, Bochum
 *
 *  This library is just a copy (subset) of the timing function in
 *  weAutSys. <br />
 *  See all else in the weAutSys documentation. <br />
 *  To do: Try to replace this copy and paste by a better approach in future.
 *
 *  $Revision: 1 $
 *  $Date: 2017-01-25 11:27:06 +0000 (Wed, 25 Jan 2017) $
 */
#ifndef TICK_TIMES_H
#define TICK_TIMES_H
#include <stdint.h>
#include <avr\io.h>
#include <avr\interrupt.h>
#include "arch/expConf.h"
#include "we-aut_sys/ll_common.h"


#if defined(TIMER0_COMPA_vect)
# define MS_TICK_vect TIMER0_COMPA_vect
# define MS_TICK_OCR OCR0A
#elif defined(TIMER0_COMP_vect)
# define MS_TICK_vect TIMER0_COMP_vect
# define MS_TICK_OCR OCR0
#endif

// The following settings are best explained in tickCalc.ods
#if F_CPU == 20000000
# define PRESC_TRIM_FAC_VAL 78
# define PRESC_SET_VAL 256
# define VCO_DEFAULT_SET 7
#elif F_CPU == 16000000
# define PRESC_TRIM_FAC_VAL 62
# define PRESC_SET_VAL 256
# define VCO_DEFAULT_SET 2
#elif F_CPU == 14745600
# define PRESC_TRIM_FAC_VAL 57
# define PRESC_SET_VAL 256
# define VCO_DEFAULT_SET 5
# define EXTRA_SLOW 2
#elif F_CPU == 8000000
# define PRESC_TRIM_FAC_VAL 31
# define PRESC_SET_VAL 256
# define VCO_DEFAULT_SET 4
#else
# if F_CPU >= 8000000
#   define PRESC_TRIM_FAC_VAL ((uint8_t)(F_CPU / 254410))
#   define PRESC_SET_VAL 256
# else
#   define PRESC_TRIM_FAC_VAL ((uint8_t)(F_CPU / 63120))
#   define PRESC_SET_VAL 64
# endif
# define VCO_DEFAULT_SET 7
#endif

/** Pre-scaler setting for millisecond clock
 *
 *  For CPU clock frequencies (F_CPU) above 8MHz 256 is used and 64 below.
 *
 *  For very slow F_CPU further pre-scaler setting values might be introduced
 *  in future.
 */
#define PRESC_SET PRESC_SET_VAL


/** Prescaled time factor to millisecond
 *
 *  By this factor the processor clock period pre-scaled by 256 will be the
 *  best <= approach to one millisecond.
 *
 *  For a 20 MHz machine the pre-sclaled clock period will be 12.8 s and
 *  this factor is 78, then. That will yield 998.4 s which can easily be
 *  slowed down by VCO to 1ms. <br />
 *  On a 16 MHz machine that will be 16s, 62 and 992s. <br />
 *  For other platforms and CPU clocks see the file tickCalc.ods.
 */
#define PRESC_TRIM_FAC PRESC_TRIM_FAC_VAL


/** The ms tick counter
 *
 *  This variable is incremented (in the tick interrupt
 *  handler \c ISR(TIMER0_COMPA_vect)) every millisecond. <br />
 *  This variable must never (!) be modified elsewhere.
 *
 *  Range: tick__1_ms counts to 255 and then to 0. The value is not to be
 *         related to any date and time.
 *
 *  Being just one byte not related to any other (time date) value
 *  reading this variable in application software is atomic.
 */
extern volatile uint8_t tick__1_ms;

/** The 12,8 s counter summand
 *
 *  Incremented in the 1ms interrupt by \ref PRESC_TRIM_FAC (or
 *  \ref PRESC_TRIM_FAC + 1 if VCO \ref slowVCOdown "slow").
 *
 *  This variable is not to be used directly by user / application software.
 *  Use it indirectly by \ref cnt12u8_8().
 */
extern volatile uint8_t cn12u8;

/** The 100 ms tick counter
 *
 *  This variable is incremented (in the tick interrupt
 *  handler \c ISR(TIMER0_COMPA_vect)) every 100 millisecond. <br />
 *  This variable must never (!) be modified elsewhere.
 *
 *  Range: tick100_ms counts to 255 and then to 0. The value is not to be
 *         related to any date and time.
 *
 *  Being just one byte not related to any other (time date) value
 *  reading this variable in application software is atomic.
 */
extern volatile uint8_t tick100_ms;


/** Trimming of timing oscillator
 *
 *  The higher the value the faster the oscillator for the system 1 ms ticks
 *  will be running (in the mean). The relation is non-linear.
 *
 *  Background: This value is used to slow down respectively lengthen every
 *  1st .. 255th millisecond tick period a tiny bit. All others will be a
 *  wee bit too fast.
 *
 *  Range: 1 .. 255 (0 would act as 256)
 *
 *  The best value for a 16 MHz machine is 2.
 *  The best setting for \ref intro_secH "weAut_01" board
 *  with a good 20 MHz Quartz for ATmega1284's processor clock is 7.
 *
 *  \showinitializer
 */
#define VCO_DEFAULT  VCO_DEFAULT_SET


/** Trimming of timing oscillator
 *
 *  This function will set the "value controlled oscillator" for the
 *  system's 1 ms tick to normal (usually best) speed.
 *
 *  \note If  \ref intro_sec "weAutSys"'s NTP client is active this
 *  function is for its exclusive use only.
 *  \sa VCO_DEFAULT
 */
void setVCOnormal(void) __attribute__ ((always_inline));

/** Trimming of timing oscillator
 *
 *  This function returns the current speed of the
 *  "value controlled oscillator" for the
 *  system's 1 ms tick. \ref VCO_DEFAULT is usually normal / best setting;
 *  higher values are faster.
 */
uint8_t getVCOsetting(void) __attribute__ ((pure));


/** Trimming of timing oscillator
 *
 *  This function will speed up the "value controlled oscillator" for the
 *  system's 1 ms tick a little bit. This is for catching up quite little
 *  differences to an externally given absolute (NTP e.g.) time.
 *
 *  Little differences in that sense are below -200 ms.
 *
 *  See the note at \ref setVCOnormal.
 *  \sa VCO_DEFAULT
 */
void speedVCOup(void) __attribute__ ((always_inline));

/** Trimming of timing oscillator
 *
 *  This function will slow down the "value controlled oscillator" for the
 *  system's 1 ms tick a little bit. This is for catching up quite little
 *  differences to an externally given absolute (NTP e.g.) time.
 *
 *  Little differences in that sense are below +100 ms.
 *
 *  See the note at \ref setVCOnormal.
 *  \sa VCO_DEFAULT
 *  \sa setVCOnormal
 */
void slowVCOdown(void) __attribute__ ((always_inline));

/** Get the 12.8 (16) s tick value (8 bit)
 *
 *  This function returns the actual 12.8 s respectively 16 s tick count.
 *  This value will wrap about every 3.27 ms on a 20 MHz machine and every
 *  4 ms at 16 MHz. One tick (increment) is 256 processor clocks.
 *
 *  It might be used for quite exact measures of software execution times the
 *  normal \ref tick__1_ms "milliseconds" resolution is of little use for.
 *
 *  Hint: 12.8 s is correct for a 20 MHz machine. For slower machines the
 *  counted period is proportionally longer. For 16 MHz it's 16 s.
 *  \sa PRESC_TRIM_FAC
 */
uint8_t cnt12u8_8(void) __attribute__ ((pure));

/** Low level clock / timer / tick initialisation
 *
 *  This function does the one time initialisation for basic timing functions.
 *  It sets the 8 bit timer (0) to provide an one ms interrupt and for fine
 *  tuning and adjusting this clock pulse in the interrupt routine
 *  (TIMER0_COMPA_vect).
 */
void initSystTiming(void); //!<  low level


/** Initialisation of the (potential) test pins
 *
 *  This function sets Port \ref setTestPin0 "C1 / testPin0" and
 *  Port \ref setTestPin1 "C0 / testPin1" as output and does
 *  \ref setTestPin0 "setTestPin0(ON)" and
 *  \ref setTestPin1 "setTestPin1(OFF)".
 *
 *  Hint: Using the testpins 0 & 1 with the module \ref intro_secH "weAut_01"
 *        inhibits the usage of Port C1 / C0 for other purposes like IC /
 *        twowire. Testpin0 JP3pin1 (would be twoWire SDA) and Testpin1 is
 *        JP3pin4 (would be SCL).
 *
 *  Hint: If the hardware is not \ref intro_secH "weAut_01" the test pins may
 *        be assigned to other ports and pins.
 *  \param TP0start initial value for Testpin 0 (On or OFF)
 *  \param TP1start initial value for Testpin 1 (On or OFF)
 */
void initTestPins(uint8_t TP0start, uint8_t TP1start);

/** Set the state of Test-Pin 0
 *
 *  This is to be used to measure the behaviour resp. timing of software
 *  by digital analysers, oscilloscopes or other instruments.
 *
 *  \param state 0 / false for off; != 0 for on
 */
void setTestPin0(uint8_t state) __attribute__((always_inline));

/** Set the state of Test-Pin 1
 *
 *  This is to be used to measure the behaviour respectively timing of
 *  software by digital analysers, oscilloscopes or other instruments.
 *
 *  \param state  0 / false for off; != 0 for on
 */
void setTestPin1(uint8_t state) __attribute__((always_inline));

#endif  // def TICK_TIMES_H
