/** \file lahn.c
 *  An application for easyAVR (7), Arduino and other supported AVR platforms
 *
 *  A little test and demo program for the TC1602B LCD module
 *
 *  Currently we support four hardware target platforms (PLATFORM=..): <br />
 - easyAVR32
 - arduMega2560
 - boAVR2013
 - arduinoUno
 *
 *  Clean and build by:   <br />
 *    make PLATFORM=easyAVR32    MAIN_F=lahn PRG_APP_P=com5 clean all
 *    make PLATFORM=boAVR2013    MAIN_F=lahn PRG_APP_P=com5 clean all
 *    make PLATFORM=arduMega2560 MAIN_F=lahn PRG_APP_P=com5 clean all
 *    make PLATFORM=arduinoUno   MAIN_F=lahn PRG_APP_P=com11 clean all
 *  depending on platform and flash by
 *    make PLATFORM=easyAVR32    MAIN_F=lahn PRG_APP_P=com5  progapp
 *    make PLATFORM=boAVR2013    MAIN_F=lahn PRG_APP_P=com5  progapp
 *    make PLATFORM=arduMega2560 MAIN_F=lahn PRG_APP_P=com5  progapp
 *    make PLATFORM=arduinoUno   MAIN_F=lahn PRG_APP_P=com11 progapp
 *  setting, of course the right serial port PRG_APP_P for the
 *  serial bootloader.
 *
 *  All else see in the include file lcd1602b.h
 *
 *  This is a test and demo application for handling a TC1602B (or akin)
 *  LCD character display. A greeting, 5 user defined characters, the revision
 *  of the serial bootloader linked to and the succession of all special
 *  character in the range 128..255 is displayed in packs of 8. The latter
 *  are different for every make and version of the 1602B module, and hardly
 *  predictable if not in full control of the ordering & manufacturing
 *  process.
 *
 *  Besides that the same UART / serial link as used by the serial bootloader
 *  is used for a greeting message and regular life sign. This is easily
 *  extended to receive commands. Optionally use a very simple serial
 *  terminal program like vsst.exe or a more comfortable one like HTerm to
 *  see / make that UART communication.
 *
 *  <br /> &nbsp;
 * \ref copyLicNote " Copyright &copy; " 2014 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 1 $
 * $Date: 2017-01-25 11:27:06 +0000 (Wed, 25 Jan 2017) $
 */
// Copyright 2014 Albrecht Weinert, Bochum

#include "arch/expConf.h"
// next 3 includes point to elements of weAutSys' serial bootloader project
#include "arch/config.h"
#include "bootloader/boot109.h"
#include "we-aut_sys/ll_common.h"

#include "lcd1602b.h"
#include "formParsUtils.h"

#include <avr/pgmspace.h>
#include <stddef.h>

uint8_t resetCauses;

/** lahn's welcome greeting and Copyright */
INFLASH(char const mainWlc[]) = "\n\n\n\n\n"
      " * * * *    AVR demo program  lahn  V1.1              * * * *  \n"
      "                                                               \n"
      " Demonstration and test for LCD modules of type TC1602B.       \n"
      " It links and uses weAutSys' serial bootloader functions.      \n"
      " Copyright (c) 2014  weinert - automation      <weAut.de>      \n"
      " Prof. Dr.-Ing. Albrecht Weinert,  Bochum  <a-weinert.de>      \n"
      "                                                               \n";


//  int main(void) __attribute__((OS_main));

/** The application (system) program start
 *
 *  This is the normal system program's program entry.
 *
 *  This application just greets the user on the same serial link (UART) as
 *  used by the serial bootloader.
 *
 *  It then acts (in a loop) as demonstration and testbed for plugable LCD
 *  displays of the TC1602B type or alike.
 */
int main(void){
   // The bootloader leaves reset causes intact if it just returns by
   // timeout. In that case it wasn't really active as bootloader.
   resetCauses = MCUSR; MCUSR = 0; // save & clear reset cause
   ///  basicSystemInit();  // done in boot loader, already

// -- first greeting via serial link (same as  for bootloader)  -----------
   sendSerBytes_P(LOW_ADD(mainWlc));          // a greeting (low flash)
   sendSerBytes_P(FAR_ADD(bootloaderPlatf)); // platform info by bootloader
   sendSerBytes_P(FAR_ADD(bootTextReset));  // bootloader function and text
   sendSerBytes_P(resetCauseText_P(resetCauses));
   sendSerByte('\n'); sendSerByte('\n');

   for(uint8_t i=0;;++i) { // endless loop          -----------------------
      if (!(i & 0x1F)) sendSerByte('~'); //  life sign via serial link
      if (i & 0x01) { // blink interval bit: life sign via blinking LED
         orPort(LIVELED_PORT, LIVELED_MASK);
      } else {
         andPort(LIVELED_PORT, ~LIVELED_MASK);
      }  // life LED on / off

// -----   LCD module initialisation  (interface, basic functions)      ----
      preInitLCDpause(SUPPLY_VOLTAGE); // long pre-init wait for 3V or 5V
      initLCD(0x08); // LCD init to Nfcb:  2line 5X8 cursorOff blinkOff

// -----   LCD module initialisation  (user def. characters, optional)   ----
      setCharPatternLCD(0, LOW_ADD(resistor));    // set pattern for 0 & 8
      setCharPatternLCD(1, LOW_ADD(capacitor));
      setCharPatternLCD(2, LOW_ADD(diode));
      setCharPatternLCD(3, LOW_ADD(inductor));
      setCharPatternLCD(4, LOW_ADD(diodeUp));
      setCharPatternLCD(5, LOW_ADD(elCo));        // set pattern for 5 & D
      cursPosLCD(0, 0);  // goto begin of line 1 & resume normal operation

// -----                01234567 8 9 x 1 2 3456   fist line              ----
      sendRawString2LCD("Hello  \0\1\2\3\4\5   ", 14); // 6 user defined char
      sendRawString2LCD_P(FAR_ADD(bootRevDat[2]), 2); // bootloader revision

// -----                                        second line  text&count  ----
      cursPosLCD(2, 0);  // goto begin of line 2
      sendData2LCD('W'); pause100ms(1); // write "World" with visible waits
      sendData2LCD('o'); pause100ms(1);
      sendData2LCD('r'); pause100ms(1);
      sendData2LCD('l'); pause100ms(1);
      sendData2LCD('d'); pause100ms(1);
      sendData2LCD(' ');
      sendData2LCD(toHexDigit(i)); // (hex) loop count 0...9, A..F
      sendData2LCD(' ');

// -----    Explore module's upper code range   second line  special ch. ----
      // Display 8 characters in the 128..255 range. The character
      // patterns are type and charge dependent and hence hardly
      // predictable if no ordering information is at hand.
      uint8_t sondZ = ((uint8_t)(i & 0x1F) * 8) | 0x80;
      sendData2LCD(sondZ);     sendData2LCD(sondZ + 1);
      sendData2LCD(sondZ + 2); sendData2LCD(sondZ + 3);
      sendData2LCD(sondZ + 4); sendData2LCD(sondZ + 5);
      sendData2LCD(sondZ + 6); sendData2LCD(sondZ + 7);

// -----    pause for humans to read the variable parts & repeat it all  ----
      pause100ms( (i&7) ?  7 : 20); // wait 2.0s at 0; 0.8s at 1..7
   } // endless for
} // main()  ("lahn")
